@@ -3,6 +3,7 @@ asynchronous core.
 
 """
 import logging
+import select
 import socket
 import time
 import warnings
@@ -17,6 +18,46 @@ from pika.adapters import base_connection
 LOGGER = logging.getLogger(__name__)
 
 
+class ReadPoller(object):
+    """A poller that will check to see if data is ready on the socket using
+    very short timeouts to avoid having to read on the socket, speeding up the
+    BlockingConnection._handle_read() method.
+
+    """
+    POLL_TIMEOUT = 0.01
+
+    def __init__(self, fd, poll_timeout=POLL_TIMEOUT):
+        """Create a new instance of the ReadPoller which wraps poll and select
+        to determine if the socket has data to read on it.
+
+        :param int fd: The file descriptor for the socket
+        :param float poll_timeout: How long to wait for events
+
+        """
+        self.fd = fd
+        self.poll_timeout = poll_timeout
+        if hasattr(select, 'poll'):
+            self.poller = select.poll()
+            self.poll_events = select.POLLIN | select.POLLPRI
+            self.poller.register(self.fd, self.poll_events)
+        else:
+            self.poller = None
+
+    def ready(self):
+        """Check to see if the socket has data to read.
+
+        :rtype: bool
+
+        """
+        if self.poller:
+            events = self.poller.poll(self.poll_timeout)
+            return True if events else False
+        else:
+            ready, unused_wri, unused_err = select.select([self.fd], [], [],
+                                                          self.poll_timeout)
+            return len(ready) > 0
+
+
 class BlockingConnection(base_connection.BaseConnection):
     """The BlockingConnection adapter is meant for simple implementations where
     you want to have blocking behavior. The behavior layered on top of the
@@ -141,6 +182,7 @@ class BlockingConnection(base_connection.BaseConnection):
         self._frames_written_without_read = 0
         self._socket_timeouts = 0
         self._timeouts = dict()
+        self._read_poller = ReadPoller(self.socket.fileno())
         self._on_connected()
         while not self.is_open:
             self.process_data_events()
@@ -181,13 +223,19 @@ class BlockingConnection(base_connection.BaseConnection):
         self._on_connection_closed(None, True)
 
     def _handle_read(self):
-        super(BlockingConnection, self)._handle_read()
-        self._frames_written_without_read = 0
+        """If the ReadPoller says there is data to read, try adn read it in the
+        _handle_read of the parent class. Once read, reset the counter that
+        keeps track of how many frames have been written since the last read.
+
+        """
+        if self._read_poller.ready():
+            super(BlockingConnection, self)._handle_read()
+            self._frames_written_without_read = 0
 
     def _handle_timeout(self):
         """Invoked whenever the socket times out"""
         self._socket_timeouts += 1
-        threshold = (self.SOCKET_CONNECT_TIMEOUT if not self.is_closing else
+        threshold = (self.SOCKET_TIMEOUT_THRESHOLD if not self.is_closing else
                      self.SOCKET_TIMEOUT_CLOSE_THRESHOLD)
 
         LOGGER.debug('Handling timeout %i with a threshold of %i',
@@ -218,6 +266,7 @@ class BlockingConnection(base_connection.BaseConnection):
         :raises: AMQPConnectionError
 
         """
+        LOGGER.info('on_connection_closed: %r, %r', method_frame, from_adapter)
         if self._is_connection_close_frame(method_frame):
             self.closing = (method_frame.method.reply_code,
                             method_frame.method.reply_text)
@@ -639,7 +688,7 @@ class BlockingChannel(channel.Channel):
 
         """
         replies = [spec.Queue.BindOk] if nowait is False else []
-        if not routing_key:
+        if routing_key is None:
             routing_key = queue
         return self._rpc(spec.Queue.Bind(0, queue, exchange, routing_key,
                                          nowait, arguments or dict()),
@@ -814,9 +863,12 @@ class BlockingChannel(channel.Channel):
 
     def _on_close(self, method_frame):
         LOGGER.warning('Received Channel.Close, closing: %r', method_frame)
-        self._send_method(spec.Channel.CloseOk(), None, False)
+        if not self.connection.is_closed:
+            self._send_method(spec.Channel.CloseOk(), None, False)
         self._set_state(self.CLOSED)
-        raise exceptions.ChannelClosed(self._reply_code, self._reply_text)
+        self._cleanup()
+        raise exceptions.ChannelClosed(method_frame.method.reply_code,
+                                       method_frame.method.reply_text)
 
     def _on_openok(self, method_frame):
         """Open the channel by sending the RPC command and remove the reply
@@ -935,6 +987,7 @@ class BlockingChannel(channel.Channel):
         self._rpc(spec.Channel.Close(self._reply_code, self._reply_text, 0, 0),
                   None,
                   [spec.Channel.CloseOk])
+        self._cleanup()
 
     def _validate_acceptable_replies(self, acceptable_replies):
         """Validate the list of acceptable replies
