@@ -365,11 +365,10 @@ class Channel(object):
         if not self.is_open:
             raise exceptions.ChannelClosed()
         LOGGER.info('Channel.close(%s, %s)', self._reply_code, self._reply_text)
-        self._set_state(self.CLOSING)
         self._reply_code, self._reply_text = reply_code, reply_text
         LOGGER.debug('Cancelling %i consumers', len(self._consumers))
         for consumer_tag in self._consumers.keys():
-            self.basic_cancel(consumer_tag)
+            self.basic_cancel(consumer_tag=consumer_tag)
         self._shutdown()
 
     def confirm_delivery(self, callback=None, nowait=False):
@@ -582,7 +581,7 @@ class Channel(object):
         """
         self._validate_channel_and_callback(callback)
         replies = [spec.Queue.BindOk] if nowait is False else []
-        if not routing_key:
+        if routing_key is None:
             routing_key = queue
         return self._rpc(spec.Queue.Bind(0, queue, exchange, routing_key,
                                          nowait, arguments or dict()), callback,
@@ -821,8 +820,10 @@ class Channel(object):
         LOGGER.warning('Received remote Channel.Close (%s): %s',
                        method_frame.method.reply_code,
                        method_frame.method.reply_text)
-        self._send_method(spec.Channel.CloseOk())
+        if self.connection.is_open:
+            self._send_method(spec.Channel.CloseOk())
         self._set_state(self.CLOSED)
+        self._cleanup()
 
     def _on_deliver(self, method_frame, header_frame, body):
         """Cope with reentrancy. If a particular consumer is still active when
@@ -931,8 +932,9 @@ class Channel(object):
         :type body: str or unicode
 
         """
-        if not self.callbacks.process(self.channel_number, '_on_return',
-                                      (self, method_frame.method,
+        if not self.callbacks.process(self.channel_number, '_on_return', self,
+                                      (self,
+                                       method_frame.method,
                                        header_frame.properties,
                                        body)):
             LOGGER.warning('Basic.Return received from server (%r, %r)',
@@ -1035,6 +1037,7 @@ class Channel(object):
         self._set_state(self.CLOSING)
         self._send_method(spec.Channel.Close(self._reply_code,
                                              self._reply_text, 0, 0))
+        self._cleanup()
 
     def _unexpected_frame(self, frame_value):
         LOGGER.warning('Unexpected frame: %r', frame_value)
@@ -1075,6 +1078,8 @@ class ContentFrameDispatcher(object):
             self._method_frame = frame_value
         elif isinstance(frame_value, frame.Header):
             self._header_frame = frame_value
+            if frame_value.body_size == 0:
+                return self._finish()
         elif isinstance(frame_value, frame.Body):
             return self._handle_body_frame(frame_value)
         else:
