@@ -190,13 +190,13 @@ class Parameters(object):
     def _validate_host(self, host):
         """Validate that the host value is an str
 
-        :param str host: The value to validate
+        :param str|unicode host: The value to validate
         :rtype: bool
         :raises: TypeError
 
         """
-        if not isinstance(host, str):
-            raise TypeError('host must be a str')
+        if not isinstance(host, basestring):
+            raise TypeError('host must be a str or unicode str')
         return True
 
     def _validate_locale(self, locale):
@@ -590,6 +590,9 @@ class Connection(object):
         # If channels are open, _on_close_ready will be called when they close
         if self._has_open_channels:
             return self._close_channels(reply_code, reply_text)
+        else:
+            self._on_close_ready()
+            return
 
     def remove_timeout(self, callback_method):
         """Adapters should override to call the callback after the
@@ -721,13 +724,13 @@ class Connection(object):
         """Add a callback for when a Connection.Tune frame is received."""
         self.callbacks.add(0, spec.Connection.Tune, self._on_connection_tune)
 
-    def _append_frame_buffer(self, bytes):
+    def _append_frame_buffer(self, value):
         """Append the bytes to the frame buffer.
 
-        :param str bytes: The bytes to append to the frame buffer
+        :param str value: The bytes to append to the frame buffer
 
         """
-        self._frame_buffer += bytes
+        self._frame_buffer += value
 
     @property
     def _buffer_size(self):
@@ -780,6 +783,8 @@ class Connection(object):
                     self._channels[channel_number].close(reply_code, reply_text)
                 else:
                     del self._channels[channel_number]
+                # Force any lingering callbacks to be removed
+                self.callbacks.cleanup(channel_number)
         else:
             self._channels = dict()
 
@@ -943,7 +948,7 @@ class Connection(object):
         self._set_connection_state(self.CONNECTION_CLOSED)
 
         # When closing, hold reason why
-        self.closing = 0, ''
+        self.closing = 0, 'Not specified'
 
         # Our starting point once connected, first frame received
         self._add_connection_start_callback()
@@ -1002,7 +1007,7 @@ class Connection(object):
         """Remove the channel from the dict of channels when Channel.CloseOk is
         sent.
 
-        :param spec.Channel.CloseOk method_frame: The response
+        :param pika.frame.Method method_frame: The response
 
         """
         LOGGER.debug('Received Channel.CloseOk')
@@ -1011,6 +1016,8 @@ class Connection(object):
         except KeyError:
             LOGGER.error('Channel %r not in channels',
                          method_frame.channel_number)
+        # Force any callbacks to be removed for this channel
+        self.callbacks.cleanup(method_frame.channel_number)
 
     def _on_close_ready(self):
         """Called when the Connection is in a state that it can close after
@@ -1309,7 +1316,8 @@ class Connection(object):
 
         """
         if self.is_closed:
-            raise exceptions.ConnectionClosed
+            LOGGER.critical('Attempted to send frame when closed')
+            return
         LOGGER.debug('Frame: %r', frame_value)
         marshaled_frame = frame_value.marshal()
         self.bytes_sent += len(marshaled_frame)
