@@ -9,8 +9,11 @@ class AMQPError(Exception):
 class AMQPConnectionError(AMQPError):
     def __repr__(self):
         if len(self.args) == 1:
-            return ('No connection could be opened after %s retries' %
-                    self.args[0])
+            if (self.args[0] == 1):
+                return ('No connection could be opened after 1 connection attempt')
+            else:
+                return ('No connection could be opened after %s connection attempts' % 
+                        self.args[0])
         elif len(self.args) == 2:
             return '%s: %s' % (self.args[0], self.args[1])
 
